/*globals require*/
require([
    'jquery',
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/presentation/ui/searchbar-test.html',
    'util/storage',

    'de_epages/presentation/ui/searchbar',
    '$ready!'
], function ($, QUnit, fixture, storage) {
    'use strict';

    QUnit.module("de_epages/presentation/ui/searchbar", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest('Checking searchbar\'s methods', 4, function () {
        $.each([
            '_create',
            '_initWidgets',
            '_initToggle',
            'destroy'
        ], function () {
            QUnit.ok(
                $.isFunction($.ui.presentationUiSearchbar.prototype[this]),
                this + ' is a method of the widget'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest('Check HTML structure and visible options', function () {

        var searchbarRoot = $('.ep-searchbar-root'),
            toggleButton,
            advancedSearch,
            quickSearch,
            toggleData;

        searchbarRoot.presentationUiSearchbar({
            statusSelect: {
              tableHeader: 'Status',
              id: 'StatusSelect',
              values:[{
                name: 'ViewedOn',
                value: 'Gesehen'
              },{
                name: 'CancelledOn',
                value: 'Abgelehnt'
              },{
                name: 'InProcessOn',
                value: 'In Bearbeitung'
              },{
                name: 'PendingOn',
                value: 'Wartet'
              },{
                name: 'ReadyForShippingOn',
                value: 'Versandfertig'
              },{
                name: 'PartlyDispatchedOn',
                value: 'Teilweise versendet'
              },{
                name: 'DispatchedOn',
                value: 'Versendet'
              },{
                name: 'ShippedOn',
                value: 'Lieferung beendet'
              },{
                name: 'PartlyInvoicedOn',
                value: 'Teilweise in Rechnung gestellt'
              },{
                name: 'InvoicedOn',
                value: 'In Rechnung gestellt'
              },{
                name: 'PartlyPaidOn',
                value: 'Teilweise bezahlt'
              },{
                name: 'Paid',
                value: 'Bezahlt'
              },{
                name: 'Returned',
                value: 'Zurückgeschickt'
              },{
                name: 'Returned',
                value: 'Zurückgeschickt'
              },{
                name: 'ClosedOn',
                value: 'Geschlossen'
              },{
                name: 'ArchivedOn',
                value: 'Archiviert'
              }]
            }
          });

        // wait for initialization
        window.setTimeout(function() {
            QUnit.ok(true, '================== [test presence of widgets] ==================');
            // check presence of searchbar widget
            QUnit.equal(
                searchbarRoot.data('uiPresentationUiSearchbar') instanceof $.Widget,
                true,
                "Searchbar widget is created and instance of jQuery.Widget"
            );

            // check presence of datepicker widget
            QUnit.equal(
                $('#CreationDateFrom').data('uiUiDatepicker') instanceof $.Widget,
                true,
                "Datepicker widget is created and instance of jQuery.Widget"
            );

            // check presence of multiselect widget
            QUnit.equal(
                $('#epMultiselect').data('uiPresentationUiMultiselect') instanceof $.Widget,
                true,
                "Multiselect widget is created and instance of jQuery.Widget"
            );

            // check presence of smartsearch widget
            QUnit.equal(
                $('#ProductAliasString').data('uiUiSmartsearch') instanceof $.Widget,
                true,
                "Smartsearch widget is created and instance of jQuery.Widget"
            );


            QUnit.ok(true, '================== [test visibility and toggle button] ==================');

            // get toggle button and its corresponding data
            toggleButton = $('button[data-id=ep-searchbar-toggle]');
            toggleData = toggleButton.data('epSearchbar');

            // get references of quick- and advancedsearch
            advancedSearch = searchbarRoot.find('[data-id=' + toggleData.options.advancedsearch + ']');
            quickSearch = searchbarRoot.find('[data-id=' + toggleData.options.quicksearch + ']');
            // remove local storage
            storage.localStorage(toggleData.options.storageId, null);

            // check initial visibility of searchbars
            QUnit.ok(
                quickSearch.is(':visible'),
                'Quicksearch is visible'
            );
            QUnit.ok(
                !advancedSearch.hasClass(':visible'),
                'Advancedsearch is not visible'
            );
            QUnit.ok(
                toggleButton.find('i.fa').hasClass('fa-chevron-down'),
                'Correct icon is displayed (fa-chevron-down)'
            );

            // trigger click on toggle button
            toggleButton.trigger('click');

            window.setTimeout(function() {
                // check visibility of searchbars after clicking toggle button
                QUnit.ok(
                    !quickSearch.is(':visible'),
                    'Quicksearch is not visible'
                );
                QUnit.ok(
                    advancedSearch.is(':visible'),
                    'Advancedsearch not visible'
                );
                QUnit.ok(
                    toggleButton.find('i.fa').hasClass('fa-chevron-up'),
                    'Correct icon is displayed (fa-chevron-up)'
                );
                QUnit.ok(
                    storage.localStorage(toggleData.options.storageId),
                    'LocalStorage was set (true)'
                );


                QUnit.ok(true, '================== [test destroy method] ==================');

                // destroy searchbar widget
                searchbarRoot.presentationUiSearchbar('destroy');

                // check presence of searchbar widget
                QUnit.equal(
                    searchbarRoot.data('uiPresentationUiSearchbar') instanceof $.Widget,
                    false,
                    "Searchbar widget was destroyed"
                );

                // check presence of datepicker widget
                QUnit.equal(
                    $('#CreationDateFrom').data('uiUiDatepicker') instanceof $.Widget,
                    false,
                    "Datepicker widget was destroyed"
                );

                // check presence of multiselect widget
                QUnit.equal(
                    $('#epMultiselect').data('uiPresentationUiMultiselect') instanceof $.Widget,
                    false,
                    "Multiselect widget was destroyed"
                );

                // check presence of smartsearch widget
                QUnit.equal(
                    $('#ProductAliasString').data('uiUiSmartsearch') instanceof $.Widget,
                    false,
                    "Smartsearch widget was destroyed"
                );

                // reset local storage
                storage.localStorage(toggleData.options.storageId, null);

                QUnit.start();
            }, 100);
        }, 100);
    });
});