/*globals define*/
/*jslint nomen:true*/

define([
    'jquery',
    'testsuite/qunit',
    'de_epages/shop/linkpicker/additionalpagesview',

    '$ready!'
], function ($, QUnit, Additionalpagesview) {
    'use strict';

    var container = $('#qunit-fixture'),
        data = {},

        advancedLinksResult = [{
            "Alias": "Advanced search",
            "Link": "?ViewAction=ViewDetailSearchProducts&ObjectPath=%2FShops%2FDemoShop"
        }, {
            "Alias": "New registration",
            "Link": "?ViewAction=ViewRegistration&ObjectPath=%2FShops%2FDemoShop"
        }, {
            "Alias": "Sign in",
            "Link": "?ViewAction=ViewRegistrationRedirect&HideNotice=1&ObjectPath=%2FShops%2FDemoShop"
        }, {
            "Alias": "Basket",
            "Link": "?ChangeAction=CreateBasket&ObjectPath=%2FShops%2FDemoShop"
        }, {
            "Alias": "Newsletters",
            "Link": "?ViewAction=ViewNewsletter&ObjectPath=%2FShops%2FDemoShop"
        }],

        _ajax = $.ajax,
        additionalpagesview = new Additionalpagesview();

    $.ajax = function (o) {

        var result = $.Deferred(),
            jqXHR = result.promise();

        if (o.data) {
            if (o.data.ViewAction === "JSONAdvancedLinks") {
                result.resolve(advancedLinksResult);
            }
        } else {
            return _ajax(o);
        }
        return result;
    };

    QUnit.module("de_epages/shop/linkpicker/additionalpagesview");

    QUnit.test("Linkpicker-additionalpages Prototype", function () {

        $.each([
            'render',
            'clearLinkData',
            'setCheckedRadio',
            '_publishLinkData',
            'getAdditionalPages'], function (i, fn) {

            QUnit.deepEqual($.isFunction(additionalpagesview[fn]), true, "Check whether " + fn + " is a function: $.isFunction( addidtionalpagesview.prototype['" + fn + "'] )");
        });
    });

    QUnit.test("Returned data and constructed UI", function () {

        additionalpagesview.on('datafetched', function () {

            QUnit.deepEqual(advancedLinksResult, additionalpagesview.data, 'Data that is returned from the server is correct');
            additionalpagesview.render();
        });

        additionalpagesview.getAdditionalPages();
        container.empty().append(additionalpagesview.el);

        QUnit.strictEqual(container.children().children(':first').hasClass('InputRow'), true, "Check if at least one radio button is present and rendered");

        $('#ep-linkpicker-additionalpages-4').trigger('click');
        additionalpagesview.clearLinkData();
        QUnit.strictEqual($('#ep-linkpicker-additionalpages-4').prop("checked"), false, "Check if all previously selected radio buttons are unchecked");

        additionalpagesview.setCheckedRadio('?ViewAction=ViewNewsletter&ObjectPath=%2FShops%2FDemoShop');
        QUnit.deepEqual($('#ep-linkpicker-additionalpages-4').prop("checked"), true, "Check if correct radio button is checked");
    });
});