/**
 * @class de_epages.externalcontent
 * 
 */

/**
 * Display the dialog to insert or change a gadget.
 * 
 * The `de_epages.externalcontent.gadgets()` method displays a dialog in which you can select a new gadget
 * or if a existing gadget is defined in the options edit this one.
 * 
 * ### Examples
 * Open dialog with standard settings.
 * 
 * JavaScript:
 * 
 *     de_epages.externalcontent.gadgets();
 * 
 * Open dialog to change gadget.
 * 
 * JavaScript:
 * 
 *     de_epages.externalcontent.gadgets({
 *       "options":{
 *         "services":["digg","delicious","stumbleupon"]
 *       },
 *       "alias":"otherbuttons",
 *       "cartridge":"externalcontent"
 *     });
 * 
 * 
 * ### Dependencies
 * 
 *  + `de_epages`
 *  + `ep.dict`
 *  + `ep`
 *  + `ep.ui.dialog`
 *  + `backbone`
 *  + `jQuery.tmpl`
 *  + `jQuery.json`
 *  + `ep.ajax`
 *  + `jQuery.metaparse`
 * 
 * @param {Object} options A map of additional options pass to the method.
 * @param {String} options.gadget A title for the dialog.
 * @param {String} options.callback The type of the dialog, which affects the icon.
 * 
 * @method gadgets
 * @static 
 * @member de_epages.externalcontent
 * 
 * @since 6.15.0
 */

/*jslint debug: true, devel: true*/
/*global define*/
define('de_epages/externalcontent/gadgetdialog', [
    './gadgetdialog/main-view'
], function (MainView) {

    'use strict';

    var mainView = new MainView();

    return {
        open: function (options) {
            mainView.open(options);
        }
    };

});